/* This file is part of nanolisp, a very small Lisp system.

Copyright (C) 2007  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef NANOLISP_GC_H_
#define NANOLISP_GC_H_

#include "gc.h"

/* This should be called once before ever using nanolisp_make_buffer(): */
void initialize_nanolisp_garbage_collector(void);

/* Allocate a buffer in the garbage-collected heap. The content of
   the returned buffer is undefined: */
void* nanolisp_make_garbage_collectable_buffer(const size_t size_in_bytes);

/* Return a new nanolisp object which uninitialized type and content,
   allocated in the garbage-collected heap.
   This should *only* be used for implementing nanolisp_make_integer(),
   nanolisp_make_symbol(), etc.
   S-expressions should normally be allocated on the garbage-collected
   heap, so this uses nanolisp_make_garbage_collectable_buffer(). */
nanolisp_sexpression_t nanolisp_make_uninitialized_sexpression(void);

/* Perform a full garbage collection. This is only useful for debugging:
   of course the garbage collector is automatically invoked when needed. */
void nanolisp_garbage_collect(void);

#endif // #ifndef NANOLISP_GC_H_
