/* This file is part of nanolisp, a very small Lisp system.

Copyright (C) 2007  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef NANOLISP_INTERPRETER_H_
#define NANOLISP_INTERPRETER_H_

void initialize_interpreter(void);

/* This is the interpreter: evaluate the given expression in the
   given environment, and return the result (which is still a nanolisp
   s-expression): */
nanolisp_sexpression_t nanolisp_eval(nanolisp_sexpression_t expression,
                                     nanolisp_sexpression_t environment);

/* This is a 'helper function' for the interpreter: it applies the
   given procedure (which may be a nanolisp procedure or a primitive
   procedure implemented in C) to the given parameters.
   When passed to apply() the procedure and all its parameters are
   already evaluated, i.e. they have been returned by eval().
   The parameters are passed in a single nanolisp list. */
nanolisp_sexpression_t nanolisp_apply(nanolisp_sexpression_t procedure,
                                      nanolisp_sexpression_t parameters);

#endif // #ifndef NANOLISP_INTERPRETER_H_
